@@ -31,16 +31,17 @@ def mkdir(fn):
 class Warehouse:
     '''
     :param cachedir: cachedir
-    :param dump: A function of (obj, fp),
-        like pickle.dump, json.dump, or
-        lambda obj, fp: fp.write(obj)
+    :param serializer: A thing with dump and load attribute functions,
+        like pickle, json, yaml, dill, bson, 
+        or anything in pickle_warehouse.serializers
     '''
     def __repr__(self):
         return 'Warehouse(%s)' % repr(self.cachedir)
 
-    def __init__(self, cachedir, dump = pickle.dump):
+    def __init__(self, cachedir, serializer = pickle, mutable = True):
         self.cachedir = cachedir
-        self.dump = dump
+        self.serializer = serializer
+        self.mutable = mutable
 
     def filename(self, index):
         return os.path.join(self.cachedir, *parse_identifier(index))
@@ -51,19 +52,25 @@ class Warehouse:
     def __setitem__(self, index, obj):
         fn = self.filename(index)
         mkdir(fn)
-        with open(fn, 'wb') as fp:
-            self.dump(obj, fp)
+        if (not self.mutable) and os.path.exists(fn):
+            raise PermissionError('This warehouse is immutable, and %s already exists.' % fn)
+        else:
+            with open(fn, 'wb') as fp:
+                self.serializer.dump(obj, fp)
 
     def __getitem__(self, index):
         try:
             with open(self.filename(index), 'rb') as fp:
-                item = pickle.load(fp)
+                item = self.serializer.load(fp)
         except OpenError as e:
             raise KeyError(*e.args)
         else:
             return item
 
     def __delitem__(self, index):
+        if not self.mutable:
+            raise PermissionError('This warehouse is immutable, so you can\'t delete things.')
+
         path = self.filename(index)
         try:
             os.remove(path)
